{ RRRRRR                  ReportBuilder Class Library                  BBBBB
  RR   RR                                                              BB   BB
  RRRRRR                 Digital Metaphors Corporation                 BB BB
  RR  RR                                                               BB   BB
  RR   RR                   Copyright (c) 1996-2002                    BBBBB   

                  Modified with permission by Liodden Data AS                  }

unit daNDB;

interface

{$I ppIfDef.pas}

uses Classes, SysUtils, Forms, ExtCtrls, DB, Contnrs,
     ppClass, ppComm, ppDBPipe, ppDB, ppClasUt, ppTypes,
     daDB, daQueryDataView, daDataView, daPreviewDataDlg,
     NdbAsa, NdbAsaLg, NdbAsaDS, NdbApp;

type

  {NativeDB for SQL Anywhere (NDB) DataView Classes:

     1.  NDB TDataSet descendants
           - TDataSets that can be children of a DataView.
           - Override the HasParent method of TComponent to return True
           - Must be registerd with the Delphi IDE using the RegisterNoIcon procedure

       a. TdaChildNDBQuery        - TAsaDataSet descendant that can be a child of a DataView
       b. TdaChildNDBTable        - TAsaDataSet descendant that can be a child of a DataView
       c. TdaChildNDBStoredProc   - TAsaStoredProc descendant that can be a child of a DataView

     2.  TdaNDBSession
           - descendant of TppSession
           - implements GetDatabaseNames, GetTableNames, etc.

     3.  TdaNDBDataSet
          - descendant of TppDataSet
          - implements GetFieldNames for SQL

     4.  TdaNDBQueryDataView
          - descendant of TppQueryDataView
          - uses the above classes to create the required
            Query -> DataSource -> Pipeline -> Report connection
          - uses the TdaSQL object built by the QueryWizard to assign
            SQL to the TNDBQuery etc.
      }

  { TdaChildNDBQuery }
  TdaChildNDBQuery = class(TAsaDataset)
    public
      function HasParent: Boolean; override;
    end;

  { TdaChildNDBTable }
  TdaChildNDBTable = class(TAsaDataset)
    public
      function HasParent: Boolean; override;
    end;

  { TdaChildNDBStoredProc }
  TdaChildNDBStoredProc = class(TAsaStoredProc)
    public
      function HasParent: Boolean; override;
  end;

  { TdaNDBSession }
  TdaNDBSession = class(TdaSession)
    protected
      function GetDefaultDatabase(const aDatabaseName: String): TComponent; override;
      function IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean; override;
    public
      class function ClassDescription: String; override;
      class function DataSetClass: TdaDataSetClass; override;
      class function DatabaseClass: TComponentClass; override;
      class function GetDefaultNDBConnection: TAsaSession;
      class function LogonToDatabase(aConnection: TAsaSession): Boolean;
      procedure GetDatabaseNames(aList: TStrings); override;
      function  GetDatabaseType(const aDatabaseName: String): TppDatabaseType; override;
      procedure GetTableNames(const aDatabaseName: String; aList: TStrings); override;
      function  ValidDatabaseTypes: TppDatabaseTypes; override;
  end;

  { TdaNDBDataSet }
  TdaNDBDataSet = class(TdaDataSet)
    private
      FQuery: TAsaDataset;
      function GetQuery: TAsaDataset;
    protected
      procedure BuildFieldList; override;
      function  GetActive: Boolean; override;
      procedure SetActive(Value: Boolean); override;
      procedure SetDatabase(aDatabase: TComponent); override;
      procedure SetDataName(const aDataName: String); override;
      property Query: TAsaDataset read GetQuery;
    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;
      class function ClassDescription: String; override;
      procedure GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings); override;
      procedure GetFieldsForSQL(aList: TList; aSQL: TStrings); override;
  end;

  { TdaNDBQueryDataView }
  TdaNDBQueryDataView = class(TdaQueryDataView)
    private
      FDataSource: TppChildDataSource;
      FQuery: TdaChildNDBQuery;
    protected
      procedure SQLChanged; override;
    public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;
      class function PreviewFormClass: TFormClass; override;
      class function SessionClass: TClass; override;
      procedure Init; override;
      procedure ConnectPipelinesToData; override;
    published
      property DataSource: TppChildDataSource read FDataSource;
  end;

  {Delphi design time registration}
  procedure Register;

implementation

const
  cDefaultConnection = 'DefaultNDBConnection';

var
  FNDBConnection: TAsaSession;
  FNDBLogin: TAsaLogin;

{******************************************************************************
 *
 ** R E G I S T E R
 *
{******************************************************************************}

procedure Register;
begin
  {NDB DataAccess Components}
  RegisterNoIcon([TdaChildNDBQuery, TdaChildNDBTable, TdaChildNDBStoredProc]);
  {NDB DataViews}
  RegisterNoIcon([TdaNDBQueryDataView]);
end;

{******************************************************************************
 *
 ** C H I L D  N A T I V E D B  D A T A   A C C E S S   C O M P O N  E N T S
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaChildNDBQuery.HasParent }

function TdaChildNDBQuery.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildNDBTable.HasParent }

function TdaChildNDBTable.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{------------------------------------------------------------------------------}
{ TdaChildNDBStoredProc.HasParent }

function TdaChildNDBStoredProc.HasParent: Boolean;
begin
  Result := True;
end; {function, HasParent}

{******************************************************************************
 *
 ** N A T I V E D B   S E S S I O N
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaNDBSession.ClassDescription }

class function TdaNDBSession.ClassDescription: String;
begin
  Result := 'NDBSession';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaNDBSession.DataSetClass }

class function TdaNDBSession.DataSetClass: TdaDataSetClass;
begin
  Result := TdaNDBDataSet;
end; {class function, DataSetClass}

{------------------------------------------------------------------------------}
{ TdaNDBSession.DatabaseClass }

class function TdaNDBSession.DatabaseClass: TComponentClass;
begin
  Result := TAsaSession;
end;

{------------------------------------------------------------------------------}
{ TdaNDBSession.GetDefaultNDBConnection }

class function TdaNDBSession.GetDefaultNDBConnection: TAsaSession;
begin

  {create the default Connection, if needed}
  if (FNDBConnection = nil) then
    begin
      {create default NDB Connection}
      FNDBConnection := TAsaSession.Create(nil);
      FNDBConnection.Name := cDefaultConnection;

      FNDBLogin := TAsaLogin.Create(nil);
      FNDBLogin.Session := FNDBConnection;
    end;

  Result := FNDBConnection;

end; {class function, GetDefaultNDBConnection}

{------------------------------------------------------------------------------}
{ TdaNDBSession.LogonToDatabase }

class function TdaNDBSession.LogonToDatabase(aConnection: TAsaSession): Boolean;
begin

  if (aConnection = nil) then
    Result := False

  else if aConnection.Connected then
    Result := True

  else
    begin
      if (aConnection = GetDefaultNDBConnection) then
        FNDBLogin.Execute
      else
        aConnection.Connected := True;

      Result := aConnection.Connected;
    end;

end; {class function, LogonToDatabase}

{------------------------------------------------------------------------------}
{ TdaNDBSession.GetDatabaseType }

function TdaNDBSession.GetDatabaseType(const aDatabaseName: String): TppDatabaseType;
begin
  Result := dtSybaseASA;
end; {procedure, GetDatabaseType}

{------------------------------------------------------------------------------}
{ TdaNDBSession.GetTableNames }

procedure TdaNDBSession.GetTableNames(const aDatabaseName: String; aList: TStrings);
var
  lDatabase: TAsaSession;
  lsTableName: String;
  lQuery: TAsaDataset;

begin
  aList.Clear;

  {get the database}
  lDatabase := TAsaSession(GetDatabaseForName(aDatabaseName));

  {connection must be active to get table names}
  if not LogonToDatabase(lDatabase) then Exit;

  {get list of table names from a table object}
  lQuery := TAsaDataset.Create(nil);
  try
    lQuery.Session := lDatabase;
    lQuery.SQL.Text :=   'select creator, tname ' +
                         'from syscatalog ' +
                         'where (tabletype = ''TABLE'' ' +
                         'or tabletype = ''VIEW'') ' +
                         'and creator not in (''SYS'', ''dbo'', ''rs_systabgroup'')';

    lQuery.Open;

    if lQuery.Active then
      begin

        lQuery.First;

        while not lQuery.EOF do
          begin
            lsTableName := lQuery.Fields[0].AsString + '.' + lQuery.Fields[1].AsString;
            aList.Add(lsTableName);

            lQuery.Next;

          end;
      end;

  finally
    lQuery.Free;
  end;

end; {procedure, GetTableNames}

{------------------------------------------------------------------------------}
{ TdaNDBSession.GetDatabaseNames }

procedure TdaNDBSession.GetDatabaseNames(aList: TStrings);
var
  liIndex: Integer;
  NDBAliasList: TStringList;
  
begin

  {call inherited to build list of available TAsaSession components}
  inherited GetDatabaseNames(aList);

  NDBAliasList := TStringList.Create;
  try

    {call utility routine in NdbApp unit}
    NdbGetDatabaseNames(NDBAliasList);

//    if (NDBAliasList.Count <> aList.Count) then
//      raise EDatabaseError.Create('TdaNDBSession.GetDatabaseNames: inherited GetDatabaseNames found different number of TAsaSessions than NdbGetDatabaseNames.');

    {assign the strings, do not use aList.Assign() because aList.Objects array will get cleared}
    for liIndex := 0 to aList.Count - 1 do
      aList[liIndex] := NDBAliasList[liIndex];

  finally
    NDBAliasList.Free;
  end;
end; {procedure, GetDatabaseNames}

{------------------------------------------------------------------------------}
{ TdaNDBSession.GetDefaultDatabase }

function TdaNDBSession.GetDefaultDatabase(const aDatabaseName: String): TComponent;
var
  lAsaSession: TAsaSession;
begin

  {use the default database object}
  lAsaSession := GetDefaultNDBConnection;

  {set DatabaseName property, if needed}
  if (lAsaSession.LoginDatabase <> aDatabaseName) or (aDatabaseName = '') then
    begin
      if lAsaSession.Connected then
        lAsaSession.Connected := False;
      lAsaSession.LoginDatabase := aDatabaseName;

      {NDB does not auto logon, so force a logon dialog here}
      LogonToDatabase(lAsaSession);
    end;

  Result := lAsaSession;

end; {function, GetDefaultDatabase}

{------------------------------------------------------------------------------}
{ TdaNDBSession.IsNamedDatabase }

function TdaNDBSession.IsNamedDatabase(const aDatabaseName: String; aDatabase: TComponent): Boolean;
begin

   Result := (AnsiCompareStr(aDatabase.Name, aDatabaseName) = 0) or
             (AnsiCompareStr(TAsaSession(aDatabase).LoginDatabase, aDatabaseName) = 0);

end; {function, IsNamedDatabase}

{------------------------------------------------------------------------------}
{ TdaNDBSession.ValidDatabaseTypes }

function TdaNDBSession.ValidDatabaseTypes: TppDatabaseTypes;
begin
  Result := [dtSybaseASA];

end; {function, ValidDatabaseTypes}

{******************************************************************************
 *
 ** N A T I V E D B   D A T A S E T
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.Create }

constructor TdaNDBDataSet.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  FQuery := nil;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.Destroy }

destructor TdaNDBDataSet.Destroy;
begin

  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.ClassDescription }

class function TdaNDBDataSet.ClassDescription: String;
begin
  Result := 'NDBDataSet';
end; {class function, ClassDescription}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.GetActive }

function TdaNDBDataSet.GetActive: Boolean;
begin
  Result := GetQuery.Active
end; {function, GetActive}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.SetActive }

procedure TdaNDBDataSet.SetActive(Value: Boolean);
begin
  GetQuery.Active := Value;
end; {procedure, SetActive}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.GetQuery }

function TdaNDBDataSet.GetQuery: TAsaDataset;
begin

  {create NDB DataSet, if needed}
  if (FQuery = nil) then
    FQuery := TAsaDataset.Create(Self);

  Result := FQuery;

end; {procedure, GetQuery}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.SetDatabase }

procedure TdaNDBDataSet.SetDatabase(aDatabase: TComponent);
begin

  inherited SetDatabase(aDatabase);

  {table cannot be active to set database property}
  if GetQuery.Active then
    FQuery.Active := False;

  {set FQuery.Session}
  FQuery.Session := TAsaSession(aDatabase);

end; {procedure, SetDatabase}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.SetDataName }

procedure TdaNDBDataSet.SetDataName(const aDataName: String);
begin

  inherited SetDataName(aDataName);

  {dataset cannot be active to set data name}
  if GetQuery.Active then
    FQuery.Active := False;

  {construct an SQL statment that returns an empty result set,
   this is used to get the field information }
  FQuery.SQL.Text := 'SELECT * FROM ' + aDataName + ' WHERE ''c'' <> ''c'' ';

end; {procedure, SetDataName}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.BuildFieldList }

procedure TdaNDBDataSet.BuildFieldList;
var
  liIndex: Integer;
  lQueryField: TField;
  lField: TppField;
begin

  inherited BuildFieldList;

  {set table to active}
  if not(GetQuery.Active) then
    FQuery.Active := True;

  {create TppField objects for each field in the table}
  for liIndex := 0 to FQuery.FieldCount - 1 do
    begin
      lQueryField := FQuery.Fields[liIndex];

      lField := TppField.Create(nil);

      lField.TableName := DataName;
      lField.FieldName := lQueryField.FieldName;
      lField.FieldAlias := lQueryField.DisplayLabel;
      lField.FieldLength := lQueryField.Size;
      lField.DataType := ppConvertFieldType(lQueryField.DataType);
      lField.DisplayWidth := lQueryField.DisplayWidth;

      AddField(lField);
    end;

end; {function, BuildFieldList}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.GetFieldNamesForSQL }

procedure TdaNDBDataSet.GetFieldNamesForSQL(aList: TStrings; aSQL: TStrings);
var
  lQuery: TAsaDataset;
begin
  aList.Clear;

  {create a temporary query}
  lQuery := TAsaDataset.Create(Self);
  try

    {set the database and SQL properties}
    lQuery.Session := TAsaSession(Database);
    lQuery.SQL := aSQL;

    {get the field names}
    lQuery.GetFieldNames(aList);

  finally
    lQuery.Free;
  end;

end; {procedure, GetFieldNamesForSQL}

{------------------------------------------------------------------------------}
{ TdaNDBDataSet.GetFieldsForSQL }

procedure TdaNDBDataSet.GetFieldsForSQL(aList: TList; aSQL: TStrings);
var
  lQuery: TAsaDataset;
  lQueryField: TNativeFieldDef;
  lField: TppField;
  liIndex: Integer;
  lFieldType: TFieldType;
  lFieldSize: Word;
  lDefDisplayWidth: Integer;
begin
  aList.Clear;

  {create a temporary query}
  lQuery := TAsaDataset.Create(Self);
  try
    {assign database and SQL properties}
    lQuery.Session := TAsaSession(Database);
    lQuery.SQL := aSQL;

    {describe query}
    lQuery.Prepare;

    {create a TppField object for each field in the query}
    for liIndex := 0 to lQuery.NativeSQL.FieldDefs.Count - 1 do
      begin
        lQueryField := lQuery.NativeSQL.FieldDefs[liIndex];

        lFieldType := ftUnknown;
        lFieldSize := 0;
        lDefDisplayWidth := 10;
        case lQueryField.DataType of
          ntString:
            begin
              lFieldType := ftString;
              lFieldSize := lQueryField.Size - 1;
              lDefDisplayWidth := lFieldSize;
            end;
          ntInteger:  lFieldType := ftInteger;
          ntInt64:    lFieldType := ftFloat;
          ntFloat:    lFieldType := ftFloat;
          ntCurrency: lFieldType := ftCurrency;
          ntDate:     lFieldType := ftDate;
          ntTime:     lFieldType := ftTime;
          ntDateTime: lFieldType := ftDateTime;
          ntBoolean:  lFieldType := ftBoolean;
          ntBinary:
            begin
              lFieldType := ftBytes;
              lFieldSize := lQueryField.Size - SizeOf(Word);
              lDefDisplayWidth := lFieldSize;
            end;
          ntMemo:     lFieldType := ftMemo;
          ntBlob:     lFieldType := ftBlob;
        end;

        lField := TppField.Create(nil);

        lField.FieldName := lQueryField.Name;
        lField.FieldAlias := lQueryField.Name;
        lField.FieldLength := lFieldSize;
        lField.DataType := ppConvertFieldType(lFieldType);
        lField.DisplayWidth := lDefDisplayWidth;

        aList.Add(lField);
      end;

  finally
    lQuery.Free;
  end;

end; {procedure, GetFieldsForSQL}

{******************************************************************************
 *
 ** N A T I V E D B   Q U E R Y   D A T A V I E W
 *
{******************************************************************************}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.Create }

constructor TdaNDBQueryDataView.Create(aOwner: TComponent);
begin

  inherited Create(aOwner);

  {notes: 1. must use ChildQuery, ChildDataSource, ChildPipeline etc.
          2. use Self as owner for Query, DataSource etc.
          3. do NOT assign a Name }

  FQuery := TdaChildNDBQuery.Create(Self);

  FDataSource := TppChildDataSource.Create(Self);
  FDataSource.DataSet := FQuery;

end; {constructor, Create}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.Destroy }

destructor TdaNDBQueryDataView.Destroy;
begin
  FDataSource.Free;
  FQuery.Free;

  inherited Destroy;

end; {destructor, Destroy}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.SessionClass }

class function TdaNDBQueryDataView.SessionClass: TClass;
begin
  Result := TdaNDBSession;
end; {class function, SessionClass}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.PreviewFormClass }

class function TdaNDBQueryDataView.PreviewFormClass: TFormClass;
begin
  Result := TFormClass(GetClass('TdaPreviewDataDialog'));
end; {class function, PreviewFormClass}

{------------------------------------------------------------------------------}
{ TdaIBOQueryDataView.ConnectPipelinesToData }

procedure TdaNDBQueryDataView.ConnectPipelinesToData;
begin

  if DataPipelineCount = 0 then Exit;

  {need to reconnect here}
  TppDBPipeline(DataPipelines[0]).DataSource := FDataSource;


end; {procedure, ConnectPipelinesToData}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.Init }

procedure TdaNDBQueryDataView.Init;
var
  lDataPipeline: TppChildDBPipeline;

begin

  inherited Init;

  if DataPipelineCount > 0 then Exit;

  {note: DataView's owner must own the DataPipeline }
  lDataPipeline := TppChildDBPipeline(ppComponentCreate(Self, TppChildDBPipeline));
  lDataPipeline.DataSource := FDataSource;
 
  lDataPipeline.AutoCreateFields := False;

  {add DataPipeline to the dataview }
  lDataPipeline.DataView := Self;

end; {procedure, Init}

{------------------------------------------------------------------------------}
{ TdaNDBQueryDataView.SQLChanged }

procedure TdaNDBQueryDataView.SQLChanged;
var
  lDatabase: TComponent;
begin

  if FQuery.Active then
    FQuery.Close;

  lDatabase := Session.GetDatabaseForName(SQL.DatabaseName);

  FQuery.Session := TAsaSession(lDatabase);
  FQuery.SQL := SQL.MagicSQLText;

end; {procedure, SQLChanged}

initialization

  {register the NDB descendant classes}
  RegisterClasses([TdaChildNDBQuery, TdaChildNDBTable, TdaChildNDBStoredProc]);

  {register DADE descendant session, dataset, dataview}
  daRegisterSession(TdaNDBSession);
  daRegisterDataSet(TdaNDBDataSet);
  daRegisterDataView(TdaNDBQueryDataView);

  {initialize internal reference variables}
  FNDBConnection     := nil;
  FNDBLogin          := nil;

finalization

  {free the default connection object}
  FNDBConnection.Free;
  FNDBConnection := nil;
  FNDBLogin.Free;
  FNDBLogin := nil;

  {unregister the NDB descendant classes}
  UnRegisterClasses([TdaChildNDBQuery, TdaChildNDBTable, TdaChildNDBStoredProc]);

  {unregister DADE descendant the session, dataset, dataview}
  daUnRegisterSession(TdaNDBSession);
  daUnRegisterDataSet(TdaNDBDataSet);
  daUnRegisterDataView(TdaNDBQueryDataView);

end.
